import { bucketName, minioClient } from '../minio.ts';
import { IncomingMessage, ServerResponse } from 'http';
import { filterKeys } from './http-proxy.ts';
import { getUserFromRequest } from '@/utils/get-user.ts';
import { UserPermission, Permission } from '@kevisual/permission';
import { getLoginUser } from '@/middleware/auth.ts';

export const aiProxy = async (
  req: IncomingMessage,
  res: ServerResponse,
  opts: {
    createNotFoundPage: (msg?: string) => any;
  },
) => {
  const { createNotFoundPage } = opts;
  const _u = new URL(req.url, 'http://localhost');

  const pathname = _u.pathname;
  const params = _u.searchParams;
  const password = params.get('p');
  const version = params.get('version') || '1.0.0';
  const { user, app } = getUserFromRequest(req);
  const objectName = pathname.replace(`/${user}/${app}/`, `${user}/${app}/${version}/`);
  try {
    const stat = await minioClient.statObject(bucketName, objectName);
    if (stat.size === 0) {
      createNotFoundPage('Invalid proxy url');
      return true;
    }
    const permissionInstance = new UserPermission({ permission: stat.metaData as Permission, owner: user });
    const loginUser = await getLoginUser(req);
    const checkPermission = permissionInstance.checkPermissionSuccess({
      username: loginUser?.tokenUser?.username || '',
      password: password,
    });
    if (!checkPermission.success) {
      return createNotFoundPage('no permission');
    }
    const filterMetaData = filterKeys(stat.metaData, ['size', 'etag', 'last-modified']);
    const contentLength = stat.size;
    const etag = stat.etag;
    const lastModified = stat.lastModified.toISOString();
    const fileName = objectName.split('/').pop();

    const objectStream = await minioClient.getObject(bucketName, objectName);
    const headers = {
      'Content-Length': contentLength,
      etag,
      'last-modified': lastModified,
      'file-name': fileName,
      ...filterMetaData,
    };

    res.writeHead(200, {
      ...headers,
    });
    objectStream.pipe(res, { end: true });
    return true;
  } catch (error) {
    console.error(`Proxy request error: ${error.message}`);
    createNotFoundPage('Invalid ai proxy url');
    return false;
  }
};
